"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PDA = void 0;
const anchor = __importStar(require("@project-serum/anchor"));
const CONFIG_SEED = "config";
const CONFIG_RENT_VAULT_SEED = "config_rent_vault";
const TRADE_SEED = "trade";
const TRADE_TOKEN_VAULT_SEED = "trade_token_vault";
const TRADE_NATIVE_VAULT_SEED = "trade_native_vault";
const ARBITRATOR_CERT_SEED = "arbitrator_cert";
const KYC_CERT_SEED = "kyc_cert";
const TOKEN_INFO_SEED = "token_info";
const OFFER_SEED = "offer";
const OFFER_TOKEN_VAULT_SEED = "offer_token_vault";
const OFFER_NATIVE_VAULT_SEED = "offer_native_vault";
class PDA {
    constructor(programId, configId) {
        this.getConfigPDA = () => {
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([anchor.utils.bytes.utf8.encode(CONFIG_SEED), anchor.utils.bytes.utf8.encode(this.configId)], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getConfigRentVaultPDA = () => {
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(CONFIG_RENT_VAULT_SEED),
                anchor.utils.bytes.utf8.encode(this.configId),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getOfferPDA = (offerID) => {
            const configPDA = this.getConfigPDA();
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(OFFER_SEED),
                configPDA.key.toBuffer(),
                offerID.toArrayLike(Buffer, "le", 8),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getTradePDA = (tradeID) => {
            const configPDA = this.getConfigPDA();
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(TRADE_SEED),
                configPDA.key.toBuffer(),
                tradeID.toArrayLike(Buffer, "le", 8),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getOfferTokenVaultPDA = (offerID) => {
            const configPDA = this.getConfigPDA();
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(OFFER_TOKEN_VAULT_SEED),
                configPDA.key.toBuffer(),
                offerID.toArrayLike(Buffer, "le", 8),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getOfferNativeVaultPDA = (offerID) => {
            const configPDA = this.getConfigPDA();
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(OFFER_NATIVE_VAULT_SEED),
                configPDA.key.toBuffer(),
                offerID.toArrayLike(Buffer, "le", 8),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getTradeTokenVaultPDA = (tradeID) => {
            const configPDA = this.getConfigPDA();
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(TRADE_TOKEN_VAULT_SEED),
                configPDA.key.toBuffer(),
                tradeID.toArrayLike(Buffer, "le", 8),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getTradeNativeVaultPDA = (tradeID) => {
            const configPDA = this.getConfigPDA();
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(TRADE_NATIVE_VAULT_SEED),
                configPDA.key.toBuffer(),
                tradeID.toArrayLike(Buffer, "le", 8),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getArbitratorCertPDA = (arbitratorAuthority) => {
            const configPDA = this.getConfigPDA();
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(ARBITRATOR_CERT_SEED),
                configPDA.key.toBuffer(),
                arbitratorAuthority.toBuffer(),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getKycCertPDA = (tokenMint, user) => {
            const configPDA = this.getConfigPDA();
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(KYC_CERT_SEED),
                configPDA.key.toBuffer(),
                tokenMint.toBuffer(),
                user.toBuffer(),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getTokenInfoPDA = (tokenMint) => {
            const configPDA = this.getConfigPDA();
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(TOKEN_INFO_SEED),
                configPDA.key.toBuffer(),
                tokenMint.toBuffer(),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.programId = programId;
        this.configId = configId;
    }
}
exports.PDA = PDA;
