/// <reference types="bn.js" />
import { Connection, PublicKey } from "@solana/web3.js";
import { EscrowFetcher as FetcherOld } from "escrow-sdk-old";
import { EscrowFetcher as Fetcher, OfferClient } from "../../index";
import { ITradeDataInfo, Version } from "../types";
import { BN } from "@project-serum/anchor";
import { Wallet } from "@project-serum/anchor/dist/cjs/provider";
import { TransactionBuilder } from "@orca-so/common-sdk";
import Decimal from "decimal.js";
export declare class EscrowRouter {
    private pubkeyOld?;
    private configIdOld?;
    private pubkey;
    private configId;
    fetcherOld?: FetcherOld;
    fetcher: Fetcher;
    constructor(connection: Connection, pubkey: PublicKey, configId: string, pubkeyOld?: PublicKey, configIdOld?: string);
    getTradesByStatus(tradeStatus: object): Promise<(ITradeDataInfo | null)[]>;
    getTradesByBuyerAddress(buyerAddress: PublicKey): Promise<(ITradeDataInfo | null)[]>;
    getTradeBySellerAddress(sellerAddress: PublicKey): Promise<(ITradeDataInfo | null)[]>;
    /**
     * a function to get trade data by trade id
     */
    getTradeDataByID(tradeId: BN, version?: Version): Promise<ITradeDataInfo | null>;
    cancelTrade(wallet: Wallet, tradeId: BN): Promise<TransactionBuilder>;
    /**
     * a function to make a new offer
     * @param wallet user wallet
     * @param tokenMint mint of token that user want to create offer with
     * @param bondType type of bond. Can either be Spl or Native. Use "BondType" value from sdk
     * @param minTradeSize minimum size of trade can be created from the offer. Should be between "0.2" and "1"
     * @param tokenAmount total amount of token that user want to create offer with
     * @param fiatAmount equivalent number of fiat amount
     * @returns
     */
    makeOffer(wallet: Wallet, tokenMint: PublicKey, bondType: object, minTradeSize: Decimal, tokenAmount: BN, fiatAmount: BN): Promise<{
        tx: TransactionBuilder;
        id: BN;
    }>;
    /**
     * a function to request create trade from an existing offer
     * @param wallet user wallet
     * @param offerId offer Id that user want to create trade with
     * @param tokenAmount amount of token that user want to trade
     * @param offerClient optional: if user already have offerClient, pass it to this function to avoid fetching data from blockchain
     * @returns { tx: TransactionBuilder; id: BN } transaction to requestCreateOffer && tradeID
     */
    requestCreateTrade(wallet: Wallet, offerId: BN, tokenAmount: BN, offerClient?: OfferClient): Promise<{
        tx: TransactionBuilder;
        id: BN;
    }>;
    /**
     * a function to request create trade from an existing offer
     * @param wallet user wallet
     * @param offerId offer Id that user want to create trade with
     * @param tokenAmount amount of token that user want to trade
     * @param offerClient optional: if user already have offerClient, pass it to this function to avoid fetching data from blockchain
     * @returns { tx: TransactionBuilder; id: BN } transaction to requestCreateOffer && tradeID
     */
    cancelBuy(wallet: Wallet, tradeId: BN): Promise<TransactionBuilder>;
    release(wallet: Wallet, tradeData: ITradeDataInfo): Promise<TransactionBuilder>;
    callArbitrator(wallet: Wallet, tradeData: ITradeDataInfo): Promise<TransactionBuilder>;
    resolveDispute(wallet: Wallet, tradeData: ITradeDataInfo, disputeAction: object, compensationAction: object): Promise<TransactionBuilder>;
}
